import os
import torch
import numpy as np
import matplotlib.pyplot as plt
import matplotlib.patches as patches
import sys
sys.path.insert(0, "../datasets")
from calipsa_pytorch_dataset import filter_small_boxes
import time

idx2name = {0: 'cyclist', 1: 'car', 2: 'truck', 3: 'person'}


def draw_proposals(torch_img, boxes_list, labels=None, colors=('r',)):
    """

    :param colors:         List
    :param torch_img:      Tensor[3 x w x h]
    :param boxes_list:     List[Tensor[L x 4]]
    :param labels:         Tensor[L]
    :return:
    """

    assert(len(boxes_list) == len(colors))
    # assert(boxes.shape[0] == labels.shape[0])
    if torch_img.shape[2] != 3:
        torch_img = torch_img.permute(1, 2, 0)

    fig, ax = plt.subplots()  # Create figure and axes
    ax.imshow(torch_img)      # Display the image

    for j, boxes in enumerate(boxes_list):
        # Create Rectangle patches
        for i in range(len(boxes)):
            ax.add_patch(patches.Rectangle((boxes[i, 0].item(), boxes[i, 1].item()),
                                           (boxes[i, 2].item() - boxes[i, 0].item()),
                                           (boxes[i, 3].item() - boxes[i, 1].item()),
                                           linewidth=1, edgecolor=colors[j], facecolor='none'))

    plt.show()
    plt.close()


def draw_predictions(torch_img, boxes_list, labels, scores, colors=('g', 'r')):
    """

    :param colors:         List
    :param torch_img:      Tensor[3 x w x h]
    :param boxes_list:     List[Tensor[L x 4]] (gt, preds)
    :param labels:         Tensor[L]
    :param scores:         Tensor[L]
    :return:
    """
    assert (len(boxes_list) == len(colors))
    # assert(boxes.shape[0] == labels.shape[0])
    if torch_img.shape[2] != 3:
        torch_img = torch_img.permute(1, 2, 0)

    fig, ax = plt.subplots()  # Create figure and axes
    ax.imshow(torch_img)  # Display the image

    for j, boxes in enumerate(boxes_list):
        # Create Rectangle patches
        for i in range(len(boxes)):
            xy = (boxes[i, 0].item(), boxes[i, 1].item())
            w = boxes[i, 2].item() - boxes[i, 0].item()
            h = boxes[i, 3].item() - boxes[i, 1].item()
            ax.add_patch(patches.Rectangle(xy, w, h, linewidth=2, edgecolor=colors[j], facecolor='none'))
            if j == 1:
                caption = [idx2name[labels[i].item()], str(round(scores[i].item() * 100, 1)) + '%']
                ax.text(*xy, ': '.join(caption), style='italic', bbox={'facecolor': 'white', 'alpha': 0.5, 'pad': 0})

    plt.show()
    plt.close()


if __name__ == '__main__':
    imgs = torch.load('../../../wsddn.pytorch/data/test_imgs_100.pt')
    proposals = np.load('./test_proposal_boxes_100.npy')
    # proposals = np.load('../../../wsddn.pytorch/data/test_proposal_boxes_100.npy')
    i = 0
    while i < 20:
        img_pair = imgs[i]
        img1 = img_pair[:3, :, :]
        img2 = img_pair[3:, :, :]
        proposal = proposals[i]
        proposal[:, 2] = proposal[:, 0] + proposal[:, 2]
        proposal[:, 3] = proposal[:, 1] + proposal[:, 3]
        proposal_filter = proposal[filter_small_boxes(proposal, min_size=100)]
        draw_proposals(img1, [proposal], colors=('b',))
        time.sleep(2)
        draw_proposals(img1, [proposal_filter], colors=('b',))
        time.sleep(2)
        # print(i, _proposal1.shape, _proposal2.shape)
        i += 1
