import torch
import torch.nn as nn
import torch.nn.functional as F


class GAM(nn.Module):
    def __init__(self, dim_in, num_class):
        self.num_class = num_class
        super().__init__()
        self.gam_conv6 = nn.Sequential(
            nn.Conv2d(dim_in, dim_in, kernel_size=1, stride=1, padding=0, bias=True),
            nn.ReLU(inplace=True))
        self.gam_conv7 = nn.Sequential(
            nn.Conv2d(dim_in, num_class, kernel_size=1, stride=1, padding=0, bias=True),
            nn.ReLU(inplace=True))
        self.gam_gap = nn.AdaptiveAvgPool2d((1, 1))  # output_size: 1x1

        self._init_weights()

        self.conf_map = None

    def _init_weights(self):
        nn.init.normal_(self.gam_conv6[0].weight, std=0.01)
        nn.init.normal_(self.gam_conv6[0].bias, std=0.01)

        nn.init.normal_(self.gam_conv7[0].weight, std=0.01)
        nn.init.normal_(self.gam_conv7[0].bias, std=0.01)

    def detectron_weight_mapping(self):
        return dict(), []

    def forward(self, x):
        attention_map = self.gam_conv6(x)
        confidence_map = self.gam_conv7(attention_map)
        raw_classification_score = self.gam_gap(confidence_map).reshape(1, -1)
        assert(raw_classification_score.shape == (1, self.num_class))
        classification_score = F.softmax(raw_classification_score, dim=1)
        self.conf_map = confidence_map.clone().squeeze()
        return attention_map, classification_score

    def get_conf_map(self):
        return self.conf_map

